@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,19 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static const char copyright[] = "\
-@(#)Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/\n\
-@(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: job_local.c,v 1.16 2013/02/05 06:14:37 babolo Exp $";
-#endif /* not lint */
+#ident "@(#) Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: job_local.c,v 1.36 2018/08/13 20:54:27 babolo Exp $"
+
+#define BLIN_COMPAT      3
+#define Bpars_COMPAT     3
+#define MULAR_COMPAT     0
+#define MIFE_COMPAT      4
+#define PGOBLIN_COMPAT   2
+#define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
+#include <sys/event.h>
+#include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sysexits.h>
@@ -46,26 +51,50 @@ static const char rcsid[] = "$Id: job_local.c,v 1.16 2013/02/05 06:14:37 babolo
 #include <multilar.h>
 #include <mife.h>
 #include "pgoblin.h"
-#include "pgob.h"
 
 static int
-pgoblin_finit(u_int32_t flags, pgoblin_jobs *this) {
+/**********************************************************************
+ **                                                                  **/
+init(u_int32_t flags) {                                             /**
+ **                                                                  **
+ **********************************************************************/
     return(EX_OK);
 }
 
 static int
-local_none(pgoblin_exenv *exenv, pgoblin_r *r) {
+/**********************************************************************
+ **                                                                  **/
+fini() {                                                            /**
+ **                                                                  **
+ **********************************************************************/
+    return(EX_OK);
+}
+
+static int
+/**********************************************************************
+ **                                                                  **/
+none(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
+ **                                                                  **
+ **********************************************************************/
     return(EX_OK);
 }
  
 static int
-pgoblin_open(pgoblin_exenv *exenv, pgoblin_r *r, void *opt) {
+/**********************************************************************
+ **                                                                  **/
+jpen(pgoblin_exenv *exenv, pgoblin_nr *rn, void *opt) {             /**
+ **                                                                  **
+ **********************************************************************/
     exenv->options->flags |= PGOBLIN_INITED;
     return(EX_OK);
 }
 
 static u_int32_t
-local_test(pgoblin_command cmd) {
+/**********************************************************************
+ **                                                                  **/
+jest(pgoblin_command cmd) {                                         /**
+ **                                                                  **
+ **********************************************************************/
     u_int32_t e;
 
     if  (cmd == PGOBLIN_fork) e = PGOBLIN_TVFORK; else e = PGOBLIN_TOFORK | PGOBLIN_TOEXIT;
@@ -73,14 +102,19 @@ local_test(pgoblin_command cmd) {
 }
 
 static int
-local_exec(pgoblin_exenv *exenv, pgoblin_r *r, char **argv) {
+/**********************************************************************
+ **                                                                  **/
+jxec(pgoblin_exenv *exenv, pgoblin_nr *rn, char **argv) {           /**
+ **                                                                  **
+ **********************************************************************/
     int ex = 0;
 
-    ifBLIN_QV5(exenv->options->flags) fprintf( stderr, "+local_exec %d %d %d %d %08X argv %s\n"
-                                      , r[PGO_COUT].r, r[PGO_CCTL].r, r[PGO_CIN].r, r[PGO_CJOB].r
-                                      , exenv->exeflags, argv ? "OK" : "NULL"
-                                      );
-    ifBLIN_QV6(exenv->options->flags) {
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    ifBLIN_QX4( "+ %d %d %d %d %08X argv %s"
+              , rn[PGO_COUT], rn[PGO_CCTL], rn[PGO_CIN], rn[PGO_CJOB]
+              , exenv->flags, argv ? "OK" : "NULL"
+              );
+    ifBLIN_QO5 {
         int z;
         for (z = 0; argv[z]; z++) fprintf(stderr, "[%d]=%s~\n", z, argv[z]);
     }
@@ -89,68 +123,71 @@ local_exec(pgoblin_exenv *exenv, pgoblin_r *r, char **argv) {
         /************************************************
          * local_exec  ,    *
          ************************************************/
-    ;   if  (dup2(IO_IN->mife->fd, fileno(stdin)) < 0) {
-    ;       ifBLIN_QV1(exenv->options->flags) warn("dup2 #4");
+    ;   if  (dup2(exenv->options->io[rn[PGO_CIN]].mife->f.fd, fileno(stdin)) < 0) {
+    ;       ifBLIN_QW0("dup2 #4");
     ;       ex = -EX_OSERR;
     ;       exit(ex);
     ;   }
-    ;   if  (dup2(IO_OUT->onu, fileno(stdout)) < 0) {
-    ;       ifBLIN_QV1(exenv->options->flags) warn("dup2 #3");
+    ;   if  (dup2(exenv->options->io[rn[PGO_COUT]].onu, fileno(stdout)) < 0) {
+    ;       ifBLIN_QW0("dup2 #3");
     ;       ex = -EX_OSERR;
     ;       exit(ex);
     ;   }
     ;   if  (exenv->options->flags & PGOBLIN_FORK_WAIT) {
-    ;       ifBLIN_QV3(exenv->options->flags)
-                fprintf(stderr, "sleep %us in fork\n", exenv->options->flags & PGOBLIN_FORK_WAIT);
+    ;       ifBLIN_QX2("sleep %us in fork", exenv->options->flags & PGOBLIN_FORK_WAIT);
     ;       sleep(exenv->options->flags & PGOBLIN_FORK_WAIT);
     ;   }
-    ;   for (g = 0; g < PGOBLIN_REGSIZE; g++) {
+    ;   for (g = 0; g < pgoblin.regsize; g++) {
 #           define R exenv->options->io[g]
-    ;       if  (R.mife && (R.mife->fd != fileno(stdin)) && (R.mife->fd != fileno(stdout))) {
-    ;           if  ((R.flags & PGOBLIN_FREEMIFE) && (ex = mife_close(R.mife)))
-                    ifBLIN_QV1(exenv->options->flags) warn("mife_close in exec");
+    ;       if  (R.mife && (R.mife->f.fd != fileno(stdin)) && (R.mife->f.fd != fileno(stdout))) {
+    ;           if  ((R.flags & PGOBLIN_FREEMIFE) && (ex = mife_fini(R.mife)))
+                    ifBLIN_QW0("mife_fini in exec");
     ;       }
     ;       if  (R.clo && (R.onu != fileno(stdin)) && (R.onu != fileno(stdout))) {
     ;           if  ((R.flags & PGOBLIN_FREEOUTS) && (ex = R.clo(R.onu)))
-                    ifBLIN_QV1(exenv->options->flags) warn("clo in exec");
+                    ifBLIN_QW0("clo in exec");
     ;       }
 #           undef R
     ;   }
     ;   execve(argv[0], argv, exenv->options->envp);
     ;   ex = EX_OSERR;
-        ifBLIN_QV1(exenv->options->flags) warn("execve %s", argv[0]);
+        ifBLIN_QW0("execve %s", argv[0]);
     }
-    ifBLIN_QV5(exenv->options->flags) fprintf( stderr, "-local_exec %d\n", ex);
+    ifBLIN_QX4("- %d", ex);
     return(ex);
+#   undef blin_internal_flags
 }
 
 static int
-local_wait(pgoblin_exenv *exenv, pgoblin_r *r) {
+/**********************************************************************
+ **                                                                  **/
+jait(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
+ **                                                                  **
+ **********************************************************************/
     int ex = EX_OK, exx = EX_OK, status;
     pgoblin_jobreg *rj;
     pgoblin_jobl *j;
     size_t i;
 
-    rj = R_JOB;
+    rj = &(exenv->options->job[rn[PGO_CJOB]]);
+#   define blin_internal_flags (rj->flags & BLIN_MASK)
     if  (rj) {
-        ifBLIN_QV5(rj->flags) fprintf(stderr, "+local_wait\n");
+        ifBLIN_QX4("+");
         for (i = 0; i <  MULAR_NEXT(rj->pids); i++) {
             j = mular_getix(rj->pids, i);
             if  (!(j->pid)) {
-                ifBLIN_QV1(rj->flags) warnx("No pid in %d", (int)j->cmdn);
+                ifBLIN_QX0("No pid in %d", (int)j->cmdn);
                 errno = EINVAL;
                 ex = EX_SOFTWARE;
             } else if ((j->cmdn >= 0) && (j->status == PGOBLIN_NOSTATUS)) {
                 if  ((ex = waitpid(j->pid, &status, 0)) < 0) {
-                    ifBLIN_QV1(rj->flags) warn("Pid %d in %d", j->pid, (int)j->cmdn);
+                    ifBLIN_QW1("Pid %d in %d", j->pid, (int)j->cmdn);
                     ex = EX_OSERR;
                 } else if (ex == j->pid) {
                     j->status = status;
-                    ifBLIN_QV4(rj->flags)
-                        fprintf( stderr
-                               , "local_wait st=%08X fl=%08X ex=%d exx=%d IFEX=%d EXST=%d\n"
-                               , status, rj->flags, ex, exx, WIFEXITED(status), WEXITSTATUS(status)
-                               );
+                    ifBLIN_QX3( " st=%08X fl=%08X ex=%d exx=%d IFEX=%d EXST=%d"
+                              , status, rj->flags, ex, exx, WIFEXITED(status), WEXITSTATUS(status)
+                              );
                     if  (  (rj->flags & PGOBLIN_ERROR_SENSOR)
                         && WIFEXITED(status)
                         && WEXITSTATUS(status)
@@ -158,29 +195,35 @@ local_wait(pgoblin_exenv *exenv, pgoblin_r *r) {
                     if  ((rj->flags & PGOBLIN_ERROR_SENSIB) && exx) break;
                     ex = EX_OK;
                 } else {
-                    ifBLIN_QV1(rj->flags) warn("Pid %d<>%d in %d", ex, j->pid, (int)j->cmdn);
+                    ifBLIN_QW0("Pid %d<>%d in %d", ex, j->pid, (int)j->cmdn);
                     ex = EX_OSERR;
         }   }   }
-        ifBLIN_QV5(rj->flags) fprintf(stderr, "-pgoblin_wait %d\n", ex);
+        ifBLIN_QX4("- %d", ex);
     }
     return(ex ? ex : exx);
+#   undef blin_internal_flags
 }
 
 static pgoblin_dir*
-local_dir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {
+/**********************************************************************
+ **                                                                  **/
+jdir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {          /**
+ **                                                                  **
+ **********************************************************************/
     pgoblin_dir *d = NULL;
     int ex = EX_OK;
     FTSENT *ent;
 
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     if  (!dir == !path) {
-        ifBLIN_QV1(exenv->options->flags) warnx("local: exactly one of dir and path should be");
+        ifBLIN_QX0("local: exactly one of dir and path should be");
         ERROUT(EX_DATAERR, EINVAL);
     }
     if  (path) {
         char *p[2] = {NULL, NULL};
 
         if  (!(d = calloc(1, sizeof(pgoblin_dir)))) {
-            ifBLIN_QV1(exenv->options->flags) warnx("local dir: no mem");
+            ifBLIN_QX0("local dir: no mem");
             ERROUT(EX_OSERR, ENOMEM);
         }
         d->i = 0;
@@ -188,7 +231,7 @@ local_dir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {
         d->g = 1;
         p[0] = path;
         if  (!(d->dirp = fts_open(p, FTS_PHYSICAL | FTS_NOCHDIR | FTS_SEEDOT, NULL))) {
-            ifBLIN_QV1(exenv->options->flags) warnx("local dir: no fts");
+            ifBLIN_QX0("local dir: no fts");
             ERROUT(EX_OSERR, EIO);
         }
     } else {
@@ -223,21 +266,23 @@ local_dir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {
         case FTS_F:
             break;
         case FTS_DNR:
-            ifBLIN_QV2(exenv->options->flags) warnc(ent->fts_errno, "local dir: fts_read DNR");
+            errno = ent->fts_errno;
+            ifBLIN_QW1("fts_read DNR");
             break;
         case FTS_NS:
-            ifBLIN_QV2(exenv->options->flags) warnc(ent->fts_errno, "local dir: fts_read NS");
+            errno = ent->fts_errno;
+            ifBLIN_QW1("fts_read NS");
             /* FALLTHRU */
         case FTS_NSOK:
             ent->fts_statp = NULL;
             break;
         case FTS_ERR:
             errno = ent->fts_errno;
-            ifBLIN_QV1(exenv->options->flags) warn("local dir: fts_read");
+            ifBLIN_QW0("fts_read");
             ex = EX_OSERR;
             goto out;
         default:
-            ifBLIN_QV1(exenv->options->flags) warn("local dir: unknown fts_read");
+            ifBLIN_QW0("unknown fts_read");
             ERROUT(EX_OSERR, EIO);
         }
         d->sb = ent->fts_statp;
@@ -252,19 +297,23 @@ out:
             d = NULL;
     }   }
     return(d);
+#   undef blin_internal_flags
 }
 
 pgoblin_jobs pgoblin_job_local =
 { 0
 , "4pGoblin-" VERS
 , "local\0" VERS
-, pgoblin_finit
-, pgoblin_finit
-, pgoblin_open
-, local_test
-, local_none /* fork */
-, local_exec
-, local_wait
-, local_none /* close */
-, local_dir
+, 0
+, init
+, fini
+, jpen
+, jest
+, none /* jork */
+, jxec
+, jait
+, none /* jose */
+, jdir
+, NULL
+, NULL
 };
